from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER


class BasicMessage(Protocol):
    """
    The BasicMessage protocol allows one self-sovereign party send another self-sovereign party a message.
    Support for this protocol is EXPERIMENTAL.
    This protocol is not implemented in the Connect.Me app and the only way it can be used is by building the mobile app using mSDK
    or using some other Aries compatible wallet that supports BasicMessage
    """

    MSG_FAMILY = 'basicmessage'
    """the family name for the message family"""

    MSG_FAMILY_VERSION = '1.0'
    """the version for the message family"""

    # Messages
    BASIC_MESSAGE = 'send-message'
    """Name for 'send-message' control message"""

    # pylint:disable=too-many-arguments
    def __init__(self,
                 for_relationship: str,
                 thread_id: str = None,
                 content: str = None,
                 sent_time: str = '',
                 localization: str = None):
        """
        Args:
            for_relationship (str): the relationship identifier (DID) for the pairwise relationship that will be used
            thread_id (str): the thread id of the already started protocol
            content (str): The main text of the message
            sent_time (str): The time the message was sent
            localization (str): Language localization code
        """
        super().__init__(self.MSG_FAMILY,
                         self.MSG_FAMILY_VERSION,
                         msg_qualifier=COMMUNITY_MSG_QUALIFIER,
                         thread_id=thread_id)

        self.for_relationship: str = for_relationship
        self.content: str = content
        self.sent_time: str = sent_time
        self.localization: str = localization

    async def message(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.message_msg_packed(context))

    def message_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.BASIC_MESSAGE)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['content'] = self.content
        msg['sent_time'] = self.sent_time
        msg['~l10n'] = {'locale': self.localization}
        return msg

    async def message_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.message_msg(context))



from typing import Dict

from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER
from verity_sdk.utils.Exeptions import WrongSetupException


class IssueCredential(Protocol):
    """
    The IssueCredential protocol allows one self-sovereign party to issue a verifiable credential to another
    self-sovereign party. On the verity-application, these protocols use `anoncreds` as defined in the
    Hyperledger-indy project. In the verity-sdk, the interface presented by these objects allow key-value attributes
    to be issued to parties that have a DID-comm channel.
    """
    MSG_FAMILY = 'issue-credential'
    """the family name for the message family"""
    MSG_FAMILY_VERSION = '1.0'
    """the version for the message family"""

    PROPOSE = 'propose'
    """Name for 'propose' control message"""
    OFFER = 'offer'
    """Name for 'offer' control message"""
    REQUEST = 'request'
    """Name for 'request' control message"""
    ISSUE = 'issue'
    """Name for 'issue' control message"""
    REJECT = 'reject'
    """Name for 'reject' control message"""
    STATUS = 'status'
    """Name for 'status' control message"""

    SENT = 'sent'
    """Name for 'sent' signal message"""
    ACCEPT_REQUEST = 'accept-request'
    """Name for 'accept-request' signal message"""
    PROTOCOL_INVITATION = 'protocol-invitation'
    """Name for 'protocol-invitation' signal message"""

    #pylint: disable=too-many-arguments
    def __init__(self,
                 for_relationship: str,
                 thread_id: str = None,
                 cred_def_id: str = None,
                 values: Dict[str, str] = None,
                 comment: str = None,
                 price: str = '0',
                 auto_issue: bool = False,
                 by_invitation: bool = False):
        """
        Args:
            for_relationship (str): the relationship identifier (DID) for the pairwise relationship that will be used
            thread_id (str): The thread id of the already started protocol
            cred_def_id (str): the Credential Definition that will be used to issue the credential
            values (Dict[str, str]): a map of key-value pairs that make up the attributes in the credential
            comment (str): a human readable comment that is presented before issuing the credential
            price (str): (possible future feature) can and should be left un-set
            auto_issue (bool): flag to automatically issue the credential after receiving response from the receiver (skip getting
                signal for the credential request and waiting for the issue control message)
            by_invitation (bool): flag to create out-of-band invitation as a part of the IssueCredential protocol

        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=COMMUNITY_MSG_QUALIFIER,
            thread_id=thread_id
        )

        self.created = thread_id is None
        self.for_relationship = for_relationship
        self.cred_def_id = cred_def_id
        self.values = values
        self.comment = comment
        self.price = '0'  # price should be 0 regardless of what is set (until we support price)
        self.auto_issue = auto_issue
        self.by_invitation = by_invitation

    async def propose_credential(self, context):
        """
        Directs verity-application to send credential proposal.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.propose_credential_msg_packed(context))

    def propose_credential_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        if not self.created:
            raise WrongSetupException('Unable to propose credentials when NOT starting the interaction')

        msg = self._get_base_message(self.PROPOSE)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['cred_def_id'] = self.cred_def_id
        msg['credential_values'] = self.values
        msg['comment'] = self.comment
        return msg

    async def propose_credential_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.propose_credential_msg())

    async def offer_credential(self, context):
        """
        Directs verity-application to send credential offer.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.offer_credential_msg_packed(context))

    def offer_credential_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        if not self.created:
            raise WrongSetupException('Unable to offer credentials when NOT starting the interaction')

        msg = self._get_base_message(self.OFFER)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['cred_def_id'] = self.cred_def_id
        msg['credential_values'] = self.values
        msg['comment'] = self.comment
        msg['price'] = self.price
        msg['auto_issue'] = self.auto_issue
        msg['by_invitation'] = self.by_invitation
        return msg

    async def offer_credential_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.offer_credential_msg())

    async def request_credential(self, context):
        """
        Directs verity-application to send credential request.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.request_credential_msg_packed(context))

    def request_credential_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.REQUEST)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['cred_def_id'] = self.cred_def_id
        msg['comment'] = self.comment
        return msg

    async def request_credential_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.request_credential_msg())

    async def issue_credential(self, context):
        """
        Directs verity-application to issue credential and send it

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.issue_credential_msg_packed(context))

    def issue_credential_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.ISSUE)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['comment'] = self.comment
        return msg

    async def issue_credential_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.issue_credential_msg())

    async def reject(self, context):
        """
        Directs verity-application to reject the credential protocol

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.reject_msg_packed(context))

    def reject_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.REJECT)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['comment'] = self.comment
        return msg

    async def reject_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.reject_msg())

    async def status(self, context):
        """
        Ask for status from the verity-application agent

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.status_msg_packed(context))

    def status_msg(self):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.STATUS)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        return msg

    async def status_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.status_msg())



from verity_sdk.utils import EVERNYM_MSG_QUALIFIER, uuid


class MessageFamily:
    """
    Interface for message families. Message families are the set of messages used by a protocol. They include
    three types of messages: protocol messages, control messages and signal messages.

    Protocol messages are messages exchange between parties of the protocol. Each party is an independent self-sovereign
    domain.

    Control messages are messages sent by a controller (applications that use verity-sdk are controllers) to the verity
    application. These messages control the protocol and make decisions for the protocol.

    Signal messages are messages sent from the verity-application agent to a controller

    Message family messages always have a type. This type has 4 parts: qualifier, family, version and name. Three parts
    are static and defined by this interface.

    Example: did:sov:BzCbsNYhMrjHiqZDTUASHg;spec/connections/1.0/problem_report

    qualifier: did:sov:BzCbsNYhMrjHiqZDTUASHg

    family: connections

    version: 1.0

    name: problem_report
    """
    def __init__(self, msg_family: str, msg_family_version: str, msg_qualifier: str = EVERNYM_MSG_QUALIFIER):
        """
        Args:
            msg_family (str): the qualifier for the message family
            msg_family_version (str): the family name for the message family
            msg_qualifier (str): the version for the message family
        """
        self.msg_family = msg_family
        self.msg_family_version = msg_family_version
        self.msg_qualifier = msg_qualifier

    def _get_base_message(self, msg_name):
        return {
            '@id': uuid(),
            '@type': self.get_message_type(msg_name)
        }

    def message_type(self, msg_name):
        """
        Combines the element of this message family with the given message name to build a fully qualified message type

        Args:
            msg_name (str): the message name used in the built message type
        Returns:
            str: fully qualified message type
        """
        return '{};spec/{}/{}/{}'.format(self.msg_qualifier, self.msg_family, self.msg_family_version, msg_name)

    def get_message_type(self, msg_name):
        """
        Deprecated!
        See: message_type
        """
        return self.message_type(msg_name)



from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER


class UpdateConfigs(Protocol):
    """
    The UpdateConfigs protocol allow changes to the configuration of the registered agent on the verity-application.
    """

    MSG_FAMILY = 'update-configs'
    """the family name for the message family"""

    MSG_FAMILY_VERSION = '0.6'
    """the version for the message family"""

    # Messages
    UPDATE_CONFIGS = 'update'
    """Name for 'update' control message"""

    GET_STATUS = 'get-status'
    """Name for 'get-status' control message"""

    def __init__(self, name=None, logo_url=None):
        """
        Args:
            name (str): Organization's name that is presented to third-parties
            logo_url (str): A url to a logo that is presented to third-parties
        """
        super().__init__(self.MSG_FAMILY,
                         self.MSG_FAMILY_VERSION,
                         msg_qualifier=EVERNYM_MSG_QUALIFIER)
        self.name = name
        self.logo_url = logo_url

    async def update(self, context):
        """
        Directs verity-application to update the configuration register with the verity-application

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        return await self.send_message(context, await self.update_msg_packed(context))

    def update_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.UPDATE_CONFIGS)
        msg['configs'] = [
            {'name': 'name', 'value': self.name},
            {'name': 'logoUrl', 'value': self.logo_url}
        ]

        return msg

    async def update_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        msg = self.update_msg(context)
        return await self.get_message_bytes(context, msg)

    async def status(self, context):
        """
        Ask for status from the verity-application agent

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.status_msg_packed(context))

    def status_msg(self, _):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.GET_STATUS)
        return msg

    async def status_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.status_msg(context))



import json
import pytest

from verity_sdk.utils import unpack_forward_message
from verity_sdk.utils.Context import Context
from verity_sdk.protocols.Protocol import Protocol
from ..test_utils import get_test_config, cleanup


@pytest.mark.asyncio
async def test_get_message():
    message = {'hello': 'world'}
    context = await Context.create_with_config(await get_test_config())
    packed_message = await Protocol('test-family', '0.1').get_message_bytes(context, message)
    unpacked_message = json.dumps(await unpack_forward_message(context, packed_message))
    assert json.dumps(message) == unpacked_message
    await cleanup(context)



"""controls for protocols interactions on the verity-application"""
__pdoc__ = {
    'CommittedAnswer': False,
    'IssuerSetup': False,
    'UpdateConfigs': False,
    'UpdateEndpoint': False,
    'WriteCredentialDefinition': False,
    'WriteSchema': False,
}



from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER


class Connecting(Protocol):
    """
    The Connecting protocols form secure, private and self-sovereign channel between two independent parties.
    This protocol facilitates the exchange of keys and endpoints that will be used in all future interactions.
    This connecting process starts with the Relationship protocol which provisions a relationship on the Verity
    Application and creates the an invite. With the invite, this Connecting protocol can be started and completed.
    """

    MSG_FAMILY = 'connections'
    """the family name for the message family"""
    MSG_FAMILY_VERSION = '1.0'
    """the version for the message family"""

    REQUEST_RECEIVED = 'request-received'
    """Name for 'request-received' signal message"""
    RESPONSE_SENT = 'response-sent'
    """Name for 'response-sent' signal message"""
    ACCEPT_INVITE = 'accept'
    """Name for 'accept' control message"""
    STATUS = 'status'
    """Name for 'status' control message"""

    def __init__(self,
                 for_relationship: str,
                 thread_id: str = None,
                 label: str = None,
                 base64_invite_url: str = None):
        """
        Args:
            for_relationship (str): The relationship identifier (DID) to use for the connections exchange. Normally,
                its relationship will have been created as a reaction to receiving the invitation.
            thread_id (str): The thread id of the already started protocol
            label (str): A human readable string that will label the caller identity (often an organization).
                E.g. 'Acme Corp`
            base64_invite_url (str): the invitation URL as specified by the Aries 0160: Connection Protocol
                (eg. https://<domain>/<path>?c_i=<invitation-string>)
        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=COMMUNITY_MSG_QUALIFIER,
            thread_id=thread_id
        )

        self.for_relationship = for_relationship
        self.label = label
        self.base64_invite_url = base64_invite_url

    async def accept(self, context):
        """
        Accepts connection defined by the given invitation

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.accept_msg_packed(context))

    def accept_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.ACCEPT_INVITE)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        msg['label'] = self.label
        msg['invite_url'] = self.base64_invite_url
        return msg

    async def accept_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.accept_msg(context))

    async def status(self, context):
        """
        Sends the get status message to the connection

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.status_msg_packed(context))

    def status_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.STATUS)
        self._add_thread(msg)
        self._add_relationship(msg, self.for_relationship)
        return msg

    async def status_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.status_msg(context))



# This is just a reimport for backwards compatibility
# This is deprecate and will be removed in future releases of SDK
from verity_sdk.protocols.v0_6.IssuerSetup import IssuerSetup  # pylint: disable=unused-import



from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.DbcUtil import DbcUtil


class WriteSchema(Protocol):
    """
    The WriteCredentialDefinition protocol writes credential definitions to an Indy Identity Ledger
    (often the Sovrin Ledger). This protocol expect that the issuer has been setup.
    """

    MSG_FAMILY = 'write-schema'
    """the family name for the message family"""
    MSG_FAMILY_VERSION = '0.6'
    """the version for the message family"""

    # Messages
    WRITE_SCHEMA = 'write'
    """Name for 'write' control message"""

    STATUS = 'status-report'
    """Name for 'status-report' signal message"""

    def __init__(self, name: str, version: str, attrs: [str]):
        """
        Args:
            name (str): The given name for the schema
            version (str): The given version of the schema
            attrs (list[str]): A list of attribute name in the schema
        """
        super().__init__(
            self.MSG_FAMILY,
            self.MSG_FAMILY_VERSION,
            msg_qualifier=EVERNYM_MSG_QUALIFIER,
        )

        DbcUtil.require_string_not_null_or_empty(name, 'name')
        DbcUtil.require_string_not_null_or_empty(version, 'version')
        DbcUtil.require_array_not_contain_null(attrs, 'attrs')

        self.name = name
        self.version = version
        self.attrs = attrs

    async def write(self, context):
        """
        Directs verity-application to write the specified Schema to the Ledger

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent
        """
        await self.send_message(context, await self.write_msg_packed(context))

    def write_msg(self, context):
        """
        Creates the control message without packaging and sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the constructed message (dict object)
        """
        msg = self._get_base_message(self.WRITE_SCHEMA)
        msg['name'] = self.name
        msg['version'] = self.version
        msg['attrNames'] = self.attrs
        self._add_thread(msg)
        return msg

    async def write_msg_packed(self, context):
        """
        Creates and packages message without sending it.

        Args:
            context (Context): an instance of the Context object initialized to a verity-application agent

        Return:
            the bytes ready for transport
        """
        return await self.get_message_bytes(context, self.write_msg(context))



# from typing import List
#
# from verity_sdk.protocols.v1_0.CommittedAnswer import CommittedAnswer
# from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER
#
#
# class QuestionAnswer(CommittedAnswer):
#     MSG_FAMILY = 'questionanswer'
#     MSG_FAMILY_VERSION = '1.0'
#
#     def __init__(self,
#                  for_relationship: str,
#                  thread_id: str = None,
#                  question: str = None,
#                  descr: str = '',
#                  valid_responses: List[str] = None,
#                  signature_required: bool = True,
#                  answer_str=None):
#         super().__init__(for_relationship, thread_id, question, descr, valid_responses, signature_required, answer_str,
#                          self.MSG_FAMILY, self.MSG_FAMILY_VERSION, COMMUNITY_MSG_QUALIFIER)



import setuptools

from verity_sdk.version import VERSION

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name="verity-sdk",
    version=VERSION,  # see verity_sdk/version.py
    author="Evernym, Inc.",
    author_email="dev@evernym.com",
    description='The official Python SDK for Evernym\'s Verity',
    license="Apache-2.0",
    url="https://github.com/evernym/verity-sdk",
    install_requires=[
        'python3-indy~=1.15.0',
        'requests~=2.22',
        'base58~=2.0.0'
    ],
    python_requires='~=3.6',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
)




import json

import pytest

from test.test_utils import get_test_config, cleanup
from verity_sdk.protocols.v1_0.IssueCredential import IssueCredential
from verity_sdk.utils import COMMUNITY_MSG_QUALIFIER
from verity_sdk.utils.Context import Context

for_relationship = 'some_did'
comment = 'degree'
cred_def_id = '12345'
credential_values = {
    'name': 'Joe',
    'degree': 'Bachelors',
    'gpa': '3.67'
}
price = '0'
auto_issue = True
by_invitation = True
thread_id = 'some thread id'


def test_init():
    issue_credential = IssueCredential(
        for_relationship, thread_id, cred_def_id, credential_values, comment, price, auto_issue, by_invitation
    )

    assert issue_credential.for_relationship == for_relationship
    assert issue_credential.thread_id == thread_id
    assert issue_credential.cred_def_id == cred_def_id
    assert json.dumps(issue_credential.values) == json.dumps(credential_values)
    assert issue_credential.comment == comment
    assert issue_credential.price == price
    assert issue_credential.auto_issue == auto_issue
    assert issue_credential.by_invitation == by_invitation


@pytest.mark.asyncio
async def test_propose_credential():
    context = await Context.create_with_config(await get_test_config())
    issue_credential = IssueCredential(
        for_relationship, None, cred_def_id, credential_values, comment
    )
    msg = issue_credential.propose_credential_msg()

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        IssueCredential.MSG_FAMILY,
        IssueCredential.MSG_FAMILY_VERSION,
        IssueCredential.PROPOSE
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['cred_def_id'] == cred_def_id
    assert msg['credential_values']['name'] == credential_values['name']
    assert msg['credential_values']['degree'] == credential_values['degree']
    assert msg['credential_values']['gpa'] == credential_values['gpa']
    assert msg['comment'] == comment

    await cleanup(context)


@pytest.mark.asyncio
async def test_offer_credential():
    context = await Context.create_with_config(await get_test_config())
    issue_credential = IssueCredential(
        for_relationship, None, cred_def_id, credential_values, comment, price, auto_issue, by_invitation
    )
    msg = issue_credential.offer_credential_msg()

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        IssueCredential.MSG_FAMILY,
        IssueCredential.MSG_FAMILY_VERSION,
        IssueCredential.OFFER
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] is not None
    assert msg['cred_def_id'] == cred_def_id
    assert msg['credential_values']['name'] == credential_values['name']
    assert msg['credential_values']['degree'] == credential_values['degree']
    assert msg['credential_values']['gpa'] == credential_values['gpa']
    assert msg['comment'] == comment
    assert msg['price'] == price
    assert msg['auto_issue'] == auto_issue
    assert msg['by_invitation'] == by_invitation

    await cleanup(context)


@pytest.mark.asyncio
async def test_request_credential():
    context = await Context.create_with_config(await get_test_config())
    issue_credential = IssueCredential(for_relationship, thread_id, cred_def_id, None, comment)
    msg = issue_credential.request_credential_msg()

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        IssueCredential.MSG_FAMILY,
        IssueCredential.MSG_FAMILY_VERSION,
        IssueCredential.REQUEST
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] == thread_id
    assert msg['cred_def_id'] == cred_def_id
    assert msg['comment'] == comment

    await cleanup(context)


@pytest.mark.asyncio
async def test_reject_credential():
    context = await Context.create_with_config(await get_test_config())
    issue_credential = IssueCredential(for_relationship, thread_id, None, None, comment)
    msg = issue_credential.reject_msg()

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        IssueCredential.MSG_FAMILY,
        IssueCredential.MSG_FAMILY_VERSION,
        IssueCredential.REJECT
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] == thread_id
    assert msg['comment'] == comment

    await cleanup(context)

@pytest.mark.asyncio
async def test_issue_credential():
    context = await Context.create_with_config(await get_test_config())
    issue_credential = IssueCredential(for_relationship, thread_id)
    msg = issue_credential.issue_credential_msg()

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        IssueCredential.MSG_FAMILY,
        IssueCredential.MSG_FAMILY_VERSION,
        IssueCredential.ISSUE
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] == thread_id

    await cleanup(context)


@pytest.mark.asyncio
async def test_status():
    context = await Context.create_with_config(await get_test_config())
    issue_credential = IssueCredential(for_relationship, thread_id)
    msg = issue_credential.status_msg()

    assert msg['@type'] == '{};spec/{}/{}/{}'.format(
        COMMUNITY_MSG_QUALIFIER,
        IssueCredential.MSG_FAMILY,
        IssueCredential.MSG_FAMILY_VERSION,
        IssueCredential.STATUS
    )
    assert msg['@id'] is not None
    assert msg['~for_relationship'] == for_relationship
    assert msg['~thread'] is not None
    assert msg['~thread']['thid'] == thread_id

    await cleanup(context)



import requests
from verity_sdk.utils.Did import Did


async def retrieve_verity_public_did(verity_url) -> Did:
    full_url = f'{verity_url}/agency'
    result = requests.get(full_url)
    status_code = result.status_code
    if status_code > 399:
        raise IOError(f'Request failed! - {status_code} - {result.text}')

    try:
        msg = result.json()
        return Did(msg.get('DID'), msg.get('verKey'))
    except ValueError as e:
        raise IOError(f'Invalid and unexpected data from Verity -- response -- {e}')



import pytest

from test.test_utils import get_test_config, cleanup
from verity_sdk.protocols.v0_6.UpdateEndpoint import UpdateEndpoint
from verity_sdk.utils import EVERNYM_MSG_QUALIFIER
from verity_sdk.utils.Context import Context


@pytest.mark.asyncio
async def test_write():
    context = await Context.create_with_config(await get_test_config())
    update_endpoint = UpdateEndpoint()
    msg = update_endpoint.update_endpoint_msg(context)

    assert msg['@type'] == '{};spec/configs/0.6/UPDATE_COM_METHOD'.format(EVERNYM_MSG_QUALIFIER)
    assert msg['@id'] is not None
    assert msg['comMethod']['id'] == 'webhook'
    assert msg['comMethod']['type'] == 2
    assert msg['comMethod']['value'] == context.endpoint_url
    assert msg['comMethod']['packaging']['pkgType'] == '1.0'
    assert msg['comMethod']['packaging']['recipientKeys'] == [context.sdk_verkey]

    await cleanup(context)
